# -*- coding: utf-8 -*-

import time
from email import utils
import xml.etree.cElementTree as ET

from django.views.generic.base import View
from django.views.generic.list import MultipleObjectMixin
from django.contrib.sites.shortcuts import get_current_site
from django.core.urlresolvers import reverse
from django.utils.html import strip_tags
from django.utils.six import text_type, PY2
from django.http import HttpResponse

from job_offers.models import JobOffer


class XMLFeed(object):
    def __init__(self, site, objects, generation=None):
        self.site = site
        self.objects = objects
        self.generation = generation or time.time()

    def _element(self, tag, text):
        el = ET.Element(tag)
        el.text = text
        return el

    def to_element(self):
        source = ET.Element('source')
        source.append(self._element('publisher', self.site.name))
        source.append(self._element('publisherurl', 'https://{}{}'.format(self.site.domain,
                                                                          reverse('indeed-job-offer-feed'))))
        source.append(self._element('lastBuildDate', utils.formatdate(self.generation)))
        for job in self.objects:
            source.append(self._encode_job(job))
        return source

    def _format_date(self, date):
        nowtimestamp = time.mktime((date.year, date.month, date.day, 0, 0, 0, 0, 0, 0))
        return utils.formatdate(nowtimestamp)

    def _build_url(self, job):
        return 'https://{}{}'.format(self.site.domain, reverse('job-offer-detail', args=[job.slug]))

    def _build_description(self, job):
        return strip_tags(job.job_description)

    def __bytes__(self):
        return ET.tostring(self.to_element(), encoding='UTF-8')
    if PY2:
        __str__ = __bytes__

    @property
    def content_type(self):
        return 'application/xml; charset=utf-8'

    def _encode_job(self, job):
        tag = ET.Element('job')
        tag.append(self._element('title', job.title))
        tag.append(self._element('date', self._format_date(job.date)))
        tag.append(self._element('referencenumber', job.reference))
        tag.append(self._element('url', self._build_url(job)))
        tag.append(self._element('description', self._build_description(job)))

        tag.append(self._element('country', job.country or ''))
        if job.geo_area:
            tag.append(self._element('state', job.geo_area))
        if job.location:
            tag.append(self._element('city', job.location))
        if job.contract_type:
            tag.append(self._element('jobtype', job.contract_type))

        categories = [job.category, job.occupation, job.sector]
        if any(categories):
            tag.append(self._element('categories', u', '.join(text_type(category)
                                                              for category in categories
                                                              if category)))
        return tag


class IndeedXMLFeedView(MultipleObjectMixin, View):
    queryset = JobOffer.objects.filter(is_active=True)

    def get(self, request, *args, **kw):
        objects = self.get_queryset()
        site = get_current_site(self.request)
        feed = XMLFeed(site, objects)
        return HttpResponse(
            bytes(feed),
            content_type=feed.content_type
        )
